﻿using Demo.Users;

using Microsoft.EntityFrameworkCore;

using Volo.Abp.Data;
using Volo.Abp.EntityFrameworkCore;
using Volo.Abp.EntityFrameworkCore.Modeling;
using Volo.Abp.Identity;
using Volo.Abp.Identity.EntityFrameworkCore;
using Volo.Abp.Users.EntityFrameworkCore;

namespace Demo.EntityFrameworkCore
{
    /* This is your actual DbContext used on runtime.
     * It includes only your entities.
     * It does not include entities of the used modules, because each module has already
     * its own DbContext class. If you want to share some database tables with the used modules,
     * just create a structure like done for AppUser.
     *
     * Don't use this DbContext for database migrations since it does not contain tables of the
     * used modules (as explained above). See DemoMigrationsDbContext for migrations.
     */
    [ConnectionStringName("Default")]
    public class DemoDbContext : AbpDbContext<DemoDbContext>
    {
        public DbSet<AppUser> Users { get; set; }

        public DbSet<Test> Tests { get; set; }
        /* Add DbSet properties for your Aggregate Roots / Entities here.
         * Also map them inside DemoDbContextModelCreatingExtensions.ConfigureDemo
         */

        public DemoDbContext(DbContextOptions<DemoDbContext> options)
            : base(options)
        {

        }

        protected override void OnModelCreating(ModelBuilder builder)
        {
            base.OnModelCreating(builder);
            /* Configure the shared tables (with included modules) here */
            //builder.Entity<IdentityUser>().Ignore(x => x.ExtraProperties).ToTable(AbpIdentityDbProperties.DbTablePrefix + "Users");
            //builder.Entity<IdentityUserLogin>().HasKey(x => new { x.UserId, x.LoginProvider });
            //builder.Entity<IdentityUserRole>().HasKey(x => new { x.UserId, x.RoleId });
            //builder.Entity<IdentityUserToken>().HasKey(x => new { x.UserId, x.LoginProvider });
            builder.Entity<AppUser>(b =>
            {
                b.ToTable("User", AbpIdentityDbProperties.DbTablePrefix + "Users"); //Sharing the same table "AbpUsers" with the IdentityUser

                b.ConfigureByConvention();
                b.ConfigureAbpUser();

                /* Configure mappings for your additional properties
                 * Also see the DemoEfCoreEntityExtensionMappings class
                 */
            });
            builder.ConfigureIdentity();
            /* Configure your own tables/entities inside the ConfigureDemo method */

            builder.ConfigureDemo();
        }
    }
}
